/*
License: GPLv2, read more from COPYING

This file contains the functionality of discuss menu.
 */
#include <tangle.h>
#include <glib-object.h>
#include <math.h>
#include <string.h>
#include <clutter/clutter.h>
#include "discuss.h"
#include "communitymenu.h"
#include "startmenu.h"
#include "single_thread.h"
#include "community_utilities.h"
#include "profile_viewer.h"
#include "../../cem/cem.h"

static void autoscroll_to_correct_pos(ClutterActor *list_);

extern GList* paramsList;

static ClutterColor line_color = { 144, 148, 140, 255 };
static ClutterColor text_color = { 237, 28, 36, 255 };
static ClutterColor text_color2 = { 0, 0, 0, 255 };
static ClutterContainer *list = 0;
static ClutterContainer *threadslist = 0;
static gchar *entries[] = {"MUSIC MAKING", "HOBBIES", "FRIENDS", "SCHOOL", "OTHER STUFF", "ABOUT JAMMO"}; // Testing
static GList* messages = NULL;
static gint item_height = 0;
static thread_type threads[MAX_THREADS];
static int entries_size = 6;
static int threads_size = 6;
static int messages_size = 6;
static TextField titleTextField;
static TextField descriptionTextField;


void start_discuss(){
	
	cem_add_to_log("Starting Discuss", J_LOG_DEBUG);	

	ClutterActor* mainview;
	mainview = jammo_get_actor_by_id("main-views-widget");

	clutter_container_foreach(CLUTTER_CONTAINER(mainview), CLUTTER_CALLBACK(tangle_actor_hide_animated), NULL);

	ClutterActor* discuss_view = jammo_get_actor_by_id("discuss-view");

	if(discuss_view){
		clutter_actor_show(CLUTTER_ACTOR(discuss_view));
		
		if(entries_size > 5){
			list = CLUTTER_CONTAINER(jammo_get_actor_by_id("discuss-view-container"));	
		}else{
			list = CLUTTER_CONTAINER(jammo_get_actor_by_id("discuss-view-container-unscrollable"));
		}

		for(int i = 0; i < (entries_size); i++){ 
			addDiscussion(entries[i], i);	
		}

		ClutterAction* action = tangle_actor_get_action_by_type(CLUTTER_ACTOR(list), TANGLE_TYPE_SCROLL_ACTION);
		g_signal_connect_swapped(action, "clamp-offset-y", G_CALLBACK(autoscroll_to_correct_pos), list);

	}else{
		cem_add_to_log("can't find 'discuss-view' ", J_LOG_ERROR);
	}
}


gboolean addDiscussion(char* name, int number){

	gchar* arrow_image_filename = g_strdup_printf("%s/communitymenu/community_rightarrow.png", DATA_DIR);
	
	ClutterActor *line;
	ClutterActor *cont = CLUTTER_ACTOR(clutter_group_new());

	ClutterActor *arrow = tangle_button_new_with_background_actor(tangle_texture_new(
				arrow_image_filename));

	line = create_line(73, 800, 2, CLUTTER_CONTAINER(cont));

	g_free(arrow_image_filename);
        
	ClutterActor *title = clutter_text_new_full(TEXT_LARGE, name, &text_color);
	clutter_text_set_max_length(CLUTTER_TEXT(title), 15);
	clutter_actor_set_size(title, ((15*10)+30), 51); 

	ClutterActor *discussion = tangle_button_new_with_background_actor(title);	

	if(line == NULL || arrow == NULL || name == NULL)
		return 1;

	if(item_height == 0)
		item_height = 75;

        clutter_actor_set_position(arrow, 550, 12);
	clutter_container_add_actor(CLUTTER_CONTAINER(cont), arrow);
        
	clutter_container_add_actor(CLUTTER_CONTAINER(cont), discussion);
	clutter_actor_set_position(discussion, 320, 20);

	g_signal_connect_swapped(arrow, "clicked", G_CALLBACK(getDiscussion), clutter_actor_get_parent(arrow));
	g_signal_connect_swapped(CLUTTER_ACTOR(discussion), "clicked", G_CALLBACK(getDiscussion), clutter_actor_get_parent(discussion));
	
	clutter_container_add_actor(list, cont);
	
	if(number <= 6){
		clutter_actor_set_height(CLUTTER_ACTOR(list), item_height * number);
	}else{
		clutter_actor_set_height(CLUTTER_ACTOR(list), item_height * 6);
	}

	return 0;
}


gboolean addThread(char* name, int number, guint32 user_id){
	
	profile_view_params *params = malloc(sizeof(profile_view_params));
	paramsList = g_list_append(paramsList, params);
	params->user_id = user_id;
	strcpy(params->parent_view, "discuss");
	
	guint32 avatar_id = 0;
	Avatar avatar = {0,0,0,0};

	gchar* avatarbg_image_filename = g_strdup_printf("%s/communitymenu/community_avatarbackground_small.png", DATA_DIR);
	gchar* comment_image_filename = g_strdup_printf("%s/communitymenu/community_comment_icon.png", DATA_DIR);
	
	ClutterActor *line;
	ClutterActor *cont = CLUTTER_ACTOR(clutter_group_new());

	ClutterActor *avatarbg = clutter_group_new();

	clutter_container_add_actor(CLUTTER_CONTAINER(avatarbg), tangle_texture_new(avatarbg_image_filename));

	ClutterActor *starter = tangle_button_new_with_background_actor(avatarbg);

	ClutterActor *comment = tangle_button_new_with_background_actor(tangle_texture_new(comment_image_filename)); 

	line = create_line(74, 650, 1, CLUTTER_CONTAINER(cont));

	ClutterActor *text = clutter_text_new_full(TEXT_LARGE, name, &text_color2);
        clutter_actor_set_size(text, ((35*10)+30), 51);
	clutter_text_set_max_length(CLUTTER_TEXT(text), 35);
	
	avatar_id = gems_profile_manager_get_avatar_id(gems_profile_manager_get_profile_of_user(user_id));
	if (avatar_id == 0)
		avatar_id = 260; //default avatar
		
	community_get_avatar(&avatar, avatar_id);
	community_shrink_and_set_avatar(&avatar, CLUTTER_CONTAINER(avatarbg));	
	
	ClutterActor *title = tangle_button_new_with_background_actor(text);	

	if(line == NULL || comment == NULL || avatarbg == NULL|| name == NULL)
		return 1;

	if(item_height == 0)
		item_height = 75;

        clutter_actor_set_position(comment, 690, 15);
	clutter_container_add_actor(CLUTTER_CONTAINER(cont), comment);

        clutter_actor_set_position(starter, 10, 1);
	clutter_container_add_actor(CLUTTER_CONTAINER(cont), starter);

	clutter_container_add_actor(CLUTTER_CONTAINER(cont), title);
	clutter_actor_set_position(title, 80, 20);

	g_signal_connect_swapped(CLUTTER_ACTOR(title), "clicked", G_CALLBACK(getThread), clutter_actor_get_parent(title));
	g_signal_connect_swapped(comment, "clicked", G_CALLBACK(getThread), clutter_actor_get_parent(comment));
	g_signal_connect_swapped(starter, "clicked", G_CALLBACK(start_profile_view), (gpointer)params);
	
	clutter_container_add_actor(threadslist, cont);
	
	if(number <= 6){
		clutter_actor_set_height(CLUTTER_ACTOR(threadslist), item_height * number);
	}else{
		clutter_actor_set_height(CLUTTER_ACTOR(threadslist), item_height * 6); 
	}
	g_free(avatarbg_image_filename);
	g_free(comment_image_filename);

	return 0;
}


/***************************
Drawing functions
**************************/

ClutterActor *create_line(int y, int length, int size, ClutterContainer *cont) {

        ClutterActor *line = clutter_rectangle_new_with_color(&line_color);
        clutter_actor_set_size(line, length, size);
        clutter_actor_set_position(line, 0, y);

        clutter_container_add_actor(CLUTTER_CONTAINER(cont), line);
        clutter_actor_show(line);

        return line;
}

/***************************
Functions for json
**************************/


gboolean discuss_community_clicked (TangleButton* tanglebutton, gpointer none){

	end_discuss();

	startmenu_goto_communitymenu(NULL,NULL);
	return TRUE;
}

gboolean discuss_home_clicked (TangleButton* tanglebutton, gpointer none){
	
	end_discuss();

	startmenu_goto_startmenu(NULL,NULL);
	return TRUE;
}


gboolean discuss_mentor_clicked (TangleButton *tanglebutton, gpointer data)   {
	cem_add_to_log("Mentor clicked", J_LOG_DEBUG);

	return TRUE;
}

gboolean discuss_writenew_clicked (TangleButton *tanglebutton, gpointer data)   {
	cem_add_to_log("Making new thread...", J_LOG_DEBUG);

	return TRUE;
}


void end_discuss(){

	community_clear_paramslist(paramsList);

	paramsList = 0;

	clear_container(CLUTTER_CONTAINER(list));
	clear_container(CLUTTER_CONTAINER(threadslist));
	cem_add_to_log("Clear_containered containers", J_LOG_DEBUG);

}


/***************************
Callback functions
**************************/


static void autoscroll_to_correct_pos(ClutterActor *list_){

	gint slot;
	gfloat not_used, offset;
	
	ClutterAction *action = tangle_actor_get_action_by_type(list_, TANGLE_TYPE_SCROLL_ACTION);
	
	tangle_scroll_action_get_offsets(TANGLE_SCROLL_ACTION(action), &not_used, &offset);
	slot = (gint)offset % (gint)item_height < item_height/2  	? (gint)(offset) / item_height  
							  		: (gint)(offset) / item_height + 1;

	tangle_object_animate(G_OBJECT(action), CLUTTER_EASE_IN_OUT_QUAD, 300, "offset-y", 
				(gfloat)(slot*item_height), NULL);
}


void getDiscussion(ClutterActor *element){

	GList *children = clutter_container_get_children(list);
	int i = 0;
	int index_counter = 0;
	
	if(children != NULL){
		
		do{
			if(i == 0){
				i++;
			}else{	
				children = children->next;
				index_counter++;
			}
				
			if(element == children->data){
				//TODO: Check what discussion topic has been clicked and load that from db 
				discuss_change_to_discussion_threads_view(NULL, NULL);
				break;
			}
		
		}while(children->next != NULL);
	}
	g_list_free(children);
}


void getThread(ClutterActor *element){

	GList *children = clutter_container_get_children(threadslist);
	int i = 0;
	int index_counter = 0;

	getAllMessages(""); // ONLY TESTING

	if(children != NULL){
		
		do{
			if(i == 0){
				i++;
			}else{	
				children = children->next;
				index_counter++;
			}
				
			if(element == children->data){
				//TODO: Check what thread has been clicked and load that from db 
				
				clear_container(threadslist);
	
				ClutterActor* mainview = jammo_get_actor_by_id("main-views-widget");
				clutter_container_foreach(CLUTTER_CONTAINER(mainview), CLUTTER_CALLBACK(tangle_actor_hide_animated), NULL);

				start_thread_view(messages, "discuss", NULL, 0); // Testing

				break;
			}
		
		}while(children->next != NULL);
	}
	g_list_free(children);

}

gboolean add_new_thread_clicked (TangleActor *actor, gpointer data) {
	add_new_thread();
	return TRUE;
}

void add_new_thread(){
	static gboolean is_title_ok = FALSE;
	static gboolean is_msg_ok = FALSE;

	cem_add_to_log("Adding thread...", J_LOG_DEBUG);

	//ClutterActor *title = jammo_get_actor_by_id("title");
	//ClutterActor *description = jammo_get_actor_by_id("description");
	ClutterActor *error_msg = jammo_get_actor_by_id("error_message");

	if(strlen(clutter_text_get_text(CLUTTER_TEXT(titleTextField.text))) < 3){
		
		//TODO: warning that title must be at least 3 characters long
		cem_add_to_log("Title must be at least 3 characters long!", J_LOG_DEBUG);
		clutter_text_set_text(CLUTTER_TEXT(error_msg), "Title must be at least 3 characters long!");

		is_title_ok = FALSE;
	}else{
		is_title_ok = TRUE;
		cem_add_to_log("Title ok.", J_LOG_DEBUG);
	}

	if(strlen(clutter_text_get_text(CLUTTER_TEXT(descriptionTextField.text))) < 10){
		
		//TODO: warning that message must be at least 10 characters long
		cem_add_to_log("Message must be longer than 10 characters!", J_LOG_DEBUG);
		clutter_text_set_text(CLUTTER_TEXT(error_msg), "Message must be longer than 10 characters!");

		is_msg_ok = FALSE;
	}else{
		is_msg_ok = TRUE;
		cem_add_to_log("Msg ok", J_LOG_DEBUG);	
	}

	if(is_title_ok == TRUE && is_msg_ok == TRUE){

		thread_type t;
		strcpy(t.title, clutter_text_get_text(CLUTTER_TEXT(titleTextField.text)) );
		//TODO: check and handle also rest of the fields

		threads_size++;
		threads[threads_size-1] = t;

		clutter_text_set_text (CLUTTER_TEXT(jammo_get_actor_by_id("title")),"");
		clutter_text_set_text (CLUTTER_TEXT(jammo_get_actor_by_id("description")),"");
		clutter_text_set_text(CLUTTER_TEXT(error_msg), "");

		cem_add_to_log("Added thread.", J_LOG_DEBUG);
		start_discussionthreads();
	}else{
		cem_add_to_log("Cannot add thread.", J_LOG_DEBUG);
	}
}


gboolean discuss_change_to_add_thread_view(TangleActor *actor, gpointer data) {

	cem_add_to_log("changing view to add thread view", J_LOG_DEBUG);

	clear_container(threadslist);

	start_addthread_view();

	return TRUE;
}


gboolean discuss_change_to_discussion_threads_view(TangleActor *actor, gpointer data) {

	cem_add_to_log("changing view to discussion threads", J_LOG_DEBUG);

	clear_container(list);
	start_discussionthreads(); 

	return TRUE;
}


gboolean discuss_change_to_discussion_view(TangleActor *actor, gpointer data) {

	cem_add_to_log("changing view to discussion view", J_LOG_DEBUG);

	clear_container(threadslist);

	start_discuss(); 

	return TRUE;
}

gboolean start_discussionthreads_clicked(TangleActor *actor, gpointer data)  {
	start_discussionthreads();
	return TRUE;
}

void start_discussionthreads(){

	int numberOfThreads = 6;

	ClutterActor* mainview;
	mainview = jammo_get_actor_by_id("main-views-widget");

	clutter_container_foreach(CLUTTER_CONTAINER(mainview), CLUTTER_CALLBACK(tangle_actor_hide_animated), NULL); 

	ClutterActor* discussion_threads_view = jammo_get_actor_by_id("threads-view");

	//if(threads_size == 6)	
		getAllThreads();

	if(discussion_threads_view)
	{
		clutter_actor_show(CLUTTER_ACTOR(discussion_threads_view));

		if(numberOfThreads > 5)
			threadslist = CLUTTER_CONTAINER(jammo_get_actor_by_id("threads-view-list-scrollable"));	
		else
			threadslist = CLUTTER_CONTAINER(jammo_get_actor_by_id("threads-view-list-unscrollable"));
		
		int i = 0;

		for(i=0; i<threads_size; i++){
			addThread(threads[i].title, i, 0);  //TODO: get proper user_id (starter of thread)
		}

		ClutterAction* action = tangle_actor_get_action_by_type(CLUTTER_ACTOR(threadslist), TANGLE_TYPE_SCROLL_ACTION);
		g_signal_connect_swapped(action, "clamp-offset-y", G_CALLBACK(autoscroll_to_correct_pos), CLUTTER_ACTOR(threadslist));


	}else{
		cem_add_to_log("can't find 'discuss-container-for-discussion-threads' ", J_LOG_ERROR);
	}

}


void start_addthread_view(){

	ClutterActor* mainview = jammo_get_actor_by_id("main-views-widget");
	clutter_container_foreach(CLUTTER_CONTAINER(mainview), CLUTTER_CALLBACK(tangle_actor_hide_animated), NULL);
	
	ClutterActor* view = jammo_get_actor_by_id("addthread-view");	

	if(view){
		clutter_actor_show(CLUTTER_ACTOR(view));

		titleTextField.text = jammo_get_actor_by_id("title");
		titleTextField.lineCount = 1;
		titleTextField.max_characters = 50;
		titleTextField.handler_id = g_signal_connect_swapped(titleTextField.text,
							"text-changed", (GCallback)limit_line_count, &titleTextField);		

		descriptionTextField.text = jammo_get_actor_by_id("description");
		descriptionTextField.lineCount = 5;
		descriptionTextField.max_characters = 300;
		descriptionTextField.handler_id = g_signal_connect_swapped(descriptionTextField.text,
							"text-changed", (GCallback)limit_line_count, &descriptionTextField);
	}else {	
		cem_add_to_log("can't find 'addthread-view' ", J_LOG_ERROR);
	}

}


/***************************
Database functions
**************************/

void getAllThreads(){

	//TODO: get proper threads from db

	int i;

	static gboolean firstcall = TRUE;

	if(firstcall){

	// FOR TESTING ONLY
	gchar *topics[] = {"I got stuck. HELP ME!","How do I find my finished songs...","How to import samples from the web","How to make pancakes","How to win in lottery","How to learn C and clutter"}; 

	thread_type temp;
	firstcall = FALSE;

	for(i=0; i<6; i++){
		strcpy(temp.title, topics[i]);
		threads[i] = temp;
	}

	}
	
}


void getAllMessages(gchar *id){

	//TODO: get proper messages from db

	messages = NULL;

	// JUST FOR TESTING
	gchar *names[] = {"Sami", "Paavo", "Esa", "Kalle", "Pertti", "Taavi"}; 
	gchar *topics[] = {"I got stuck. HELP ME!","How do I find my finished songs...","How to import samples from the web","How to make pancakes","How to win in lottery","How to learn C and clutter"}; 

	gchar *fakemsg = "Lorem ipsum dolor sit amet, consectetuer adipiscing elit, sed diam nonummy nibh euismod tincidunt ut laoreet dolore magna aliquam erat volutpat. Ut wisi enim ad minim veniam, quis nostrud exerci tation ullamcorper suscipit lobortis nisl ut aliquip ex ea commodo consequat. Duis autem vel eum iriure dolor in hendrerit in vulputate velit esse molestie consequat, vel illum dolore eu feugiat nulla facilisis at vero eros et accumsan et iusto odio dignissim qui blandit praesent luptatum zzril delenit augue duis dolore te feugait nulla facilisi. Nam liber tempor cum soluta nobis eleifend option congue nihil imperdiet doming id quod mazim placerat facer possim assum. Typi non habent claritatem insitam; est usus legentis in iis qui facit eorum claritatem. Investigationes demonstraverunt lectores legere me lius quod ii legunt saepius. Claritas est etiam processus dynamicus, qui sequitur mutationem consuetudium lectorum. Mirum est notare quam littera gothica, quam nunc putamus parum claram, anteposuerit litterarum formas humanitatis per seacula quarta decima et quinta decima. Eodem modo typi, qui nunc nobis videntur parum clari, fiant sollemnes in futurum.";

	message *msg;
	time_type tmp;


	//if(messages_param == NULL){
		
		for(int i = 0; i < messages_size ; i++){

			msg = calloc(sizeof(message), 1);

			msg->index = 0; //Index of the message

			msg->user_id = 0; //TODO: this should be the user_id of the writer of the message
			strcpy(msg->sender, names[i]);
		
			//if(id != NULL){
				//strcpy(msg->title, id);
			
			strcpy(msg->title, topics[i]);
		
			tmp = get_current_time();
			strcpy(msg->time, tmp.time);  

			msg->msg = malloc((strlen(fakemsg)+1) *sizeof(gchar));
			strcpy(msg->msg, fakemsg); 


			messages = g_list_append(messages, (gpointer)msg);
		}	

}

void getAllTopics(){
	//TODO: get proper topics from db
}
