/*
 * community_server_functions.c
 *
 * This file is part of JamMo.
 * 
 *
 * (c) 2009-2010 University of Oulu, Lappeenranta University of Technology
 *
 * Authors: Tommi Kallonen <tommi.kallonen@lut.fi>
 */
#include "autogenerated_functions/jammo_soapH.h" // obtain the generated stub  
#include "autogenerated_functions/getJammoSongDataBinding.nsmap" // obtain the generated XML namespace mapping table for the service  
#include <glib/gstdio.h>
#include "community_server_functions.h"


gboolean getJammoData(char *songname, char *userid, char *filename, char *description, char *date, char *json)
{
	struct soap *soap = soap_new(); 
   	char *returnvalue; 
   	char ** return_;

   	return_=&returnvalue;
   	if (soap_call_ns1__getJammoData(soap, NULL, NULL, songname, userid, filename, description, date, json, return_)== SOAP_OK)
	{
		printf("Server response: %s\n",return_[0]);
		soap_end(soap);
   		soap_free(soap); 

		return TRUE;
	}
  	else // an error occurred  
	{
		soap_end(soap);
   		soap_free(soap); 

      		soap_print_fault(soap, stderr); // display the SOAP fault on the stderr stream  
		return FALSE;
	}
}

gboolean getJammoSongData(char *jammoSongID, char **songname, char **userid, char **filename, char **path, char **description, char **date, char **json)
{
	struct soap *soap = soap_new(); 
   	char *returnvalue;
   	char **return_;
	gchar **splitstring; 

   	return_=&returnvalue;
	 
   	if (soap_call_ns1__getJammoSongData(soap,NULL, NULL, jammoSongID, return_)== SOAP_OK)
	{
		printf("Server response: %s\n",return_[0]);
		splitstring = g_strsplit(return_[0], "||",-1);
		
		*songname = g_strdup(splitstring[1]);
		*userid = g_strdup(splitstring[2]);
		*filename = g_strdup(splitstring[3]);
		*path = g_strdup(splitstring[4]);
		*description = g_strdup(splitstring[5]);
		*date = g_strdup(splitstring[6]);
		*json = g_strdup(splitstring[7]);

		g_strfreev(splitstring);
		soap_end(soap);
   		soap_free(soap);

		return TRUE;
	}
   	else // an error occurred  
	{
      		soap_print_fault(soap, stderr); // display the SOAP fault on the stderr stream  

		soap_end(soap);
   		soap_free(soap);
		free(returnvalue);
   		returnvalue=NULL; 
		return FALSE;
	}
}


gboolean sendJammoFile(gchar *filename)
{
	struct soap *soap = soap_new(); 
   	char *returnvalue;
   	char ** return_;
   	gchar* filecontents=NULL;
   	gchar* filecontents64=NULL;
   	gsize length;
   	GError *file_error=NULL;

   	return_=&returnvalue;

 
	if(g_file_get_contents (filename, &filecontents, &length, &file_error))
	{

		filecontents64= g_base64_encode((const guchar *)filecontents, length);

		if (soap_call_sendfile__transfer_USCOREinkml_USCOREfile(soap, NULL, NULL, filename, filecontents64, return_)== SOAP_OK)
		{
			printf("Server response: %s\n",return_[0]);
			soap_end(soap);
   			soap_free(soap);  

   			return TRUE;
		}
   		else // an error occurred  
		{
      			soap_print_fault(soap, stderr); // display the SOAP fault on the stderr stream
			soap_end(soap);
   			soap_free(soap);  

			return FALSE;
		}

	}
   	else
   	{
		fprintf (stderr, "Unable to read file: %s\n", file_error->message);
    		g_error_free (file_error);
		soap_end(soap);
   		soap_free(soap);  

		return FALSE;
   	}

}
